// $Id: name.h,v 1.3 2019/04/15 02:35:35 leavens Exp leavens $
#ifndef NAME_H
#define NAME_H 1
#include <stdlib.h>
#include <stdbool.h>
#define MAX_NAME_LENGTH 50
// The following is for use in scanf calls
#define NAME_READING_FORMAT "%50s"

typedef struct name_s {
    const char *first_name;
    const char *last_name;
} name_t;

// Make "name" a synonym for a name_t pointer type.
typedef name_t *name;

// requires: first_name and last_name are both null-terminated
//           strings that are allocated on the heap.
// effect: creates a fresh name_t object allocated on the heap,
//         initializes its fields with the corresponding arguments,
//         and returns a pointer to the new object
//         (or NULL if the new object cannot be allocated)
//         When malloc fails the message "malloc failed!" is printed on stdout.
extern name make_name(const char *first_name, const char *last_name);

// requires: stdin and stdout are open; fn_prompt, ln_prompt are
//           all allocated, null-terminated strings.
// modifies: stdin, stdout
// effect: prompts with fn_prompt on stdout, then reads the first name.
//         Then prompts with ln_prompt on stdout and reads the last name.
//         Names are assumed to be no more than MAX_NAME_LENGTH chars long.
//         Then allocates a name_t struct on the heap and initializes it
//         with the data read from stdin, and returns a pointer to that
//         newly allocated struct (or NULL if there was an error).
//         When malloc fails the message "malloc failed!" is printed on stdout.
extern name read_name(const char *fn_prompt, const char* ln_prompt);

// requires: n is allocated (and not NULL)
// ensures: result is the first name of n
extern const char *first_name(name n);

// requires: n is allocated (and not NULL)
// ensures: result is the last name of n
extern const char *last_name(name n);

// requires: n is allocated (and not NULL)
// ensures: result is a string of the form:
//   "Firstname Lastname"
// where Firstname is the result of calling first_name() on n,
// and Lastname is the result of calling last_name() on n.
extern char *name_to_string(name n);

// requires: n1 and n2 are both allocated (and not NULL)
// ensures: result is true just when each field of n1 is equal to
// the corresnonding field of n2 (i.e., when they have the same
// first names and last names).
extern bool equal_name(name n1, name n2);
#endif
