// $Id: us_tele.h,v 1.6 2019/04/15 03:21:02 leavens Exp leavens $
#ifndef US_TELE_H
#define US_TELE_H 1
#include <stdlib.h>
#include <stdbool.h>

#define AREA_LEN 3
#define EXCHANGE_LEN 3
#define SUB_LEN 4

typedef struct {
    int area[AREA_LEN];
    int exchange[EXCHANGE_LEN];
    int subscriber[SUB_LEN];
} us_tele_t;

// Make "us_tele" a synonym for a us_tele_t pointer type.
typedef us_tele_t *us_tele;

// requires: area, exchange, and sub are all null-terminated
//           strings that are allocated, and the length of area is AREA_LEN
//           and the length of exchange is EXCHANGE_LEN, and the length
//           of sub is SUB_LEN.
// modifies: stdout
// effect: creates a fresh us_tele_t object allocated on the heap,
//         initializes its fields from the corresponding arguments,
//         and returns a pointer to the new object
//         (or NULL if the new object cannot be allocated or
//         if some of the characters in area, exchange, or sub are not digits)
//         When malloc fails the message "malloc failed!" is printed on stdout.
//         When some charactr is not a digit, a message of the form
//         "'C' is not a decimal digit" is printed on stdout, where C is the
//         non-digit character.
extern us_tele make_us_tele(const char *area, const char *exchange,
			      const char *sub);

// requires: stdin and stdout are open; area_prompt and digits_prompt are
//           both allocated, null-terminated strings.
// modifies: stdin, stdout
// effect: prompts with area_prompt on stdout, then reads the area code.
//         Then prompts with digits_prompt on stdout
//         and reads the telephone number.
//         The telephone number is expected to be in the format NNN-NNNN,
//         with a hyphen between the exchange (NNN) and the subscriber number
//         (NNNN), where each N is a decimal digit.
//         Then allocates a us_tele_t struct on the heap and initializes it
//         with the data read from stdin, and returns a pointer to that
//         newly allocated struct (or NULL if there was an error or
//         if some of the characters that should be digits are not).
//         When malloc fails the message "malloc failed!" is printed on stdout.
//         When some charactr is not a digit, a message of the form
//         "'C' is not a decimal digit" is printed on stdout, where C is the
//         non-digit character.
extern us_tele read_us_tele(const char *area_prompt, const char *digits_prompt);

// requires: te is allocated (and not NULL)
// ensures: result is a null-terminated string representing the area code of te
extern char *area_code(us_tele te);

// requires: te is allocated (and not NULL)
// ensures: result is a string of the form "Exchange-Subscriber" where
//          Exchange is a string representing te's exchange and Subscriber
//          is a string representing te's subscriber number.
extern char *digits(us_tele te);

// requires: te is allocated (and not NULL)
// ensures: result is a string of the form:
//              "(Areacode)Exchange-Subscriber"
//          where Areacode is a string representing the area code of te,
//          Exchange is a string representing te's exchange and Subscriber
//          is a string representing te's subscriber number.
extern char *us_tele_to_string(us_tele te);

// requires: te1 and te2 are both allocated (and not NULL)
// ensures: result is true just when each field of te1 is equal to
//          the corresponding field of te2.
extern bool equal_us_tele(us_tele te1, us_tele te2);
#endif
